@@ -144,7 +144,7 @@ librdf_hash_bdb_open(void* context, const char *identifier,
   DB* bdb;
   char *file;
   int ret;
-  int flags;
+  u_int32_t flags = 0;
 
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(identifier, cstring, 1);
   
@@ -160,7 +160,7 @@ librdf_hash_bdb_open(void* context, const char *identifier,
   bdb_context->is_writable=is_writable;
   bdb_context->is_new=is_new;
   
-  file=(char*)LIBRDF_MALLOC(cstring, strlen(identifier)+4);
+  file = LIBRDF_MALLOC(char*, strlen(identifier) + 4);
   if(!file)
     return 1;
   sprintf(file, "%s.db", identifier);
@@ -169,7 +169,8 @@ librdf_hash_bdb_open(void* context, const char *identifier,
   /* V3 prototype:
    * int db_create(DB **dbp, DB_ENV *dbenv, u_int32_t flags);
    */
-  if((ret=db_create(&bdb, NULL, 0))) {
+  ret = db_create(&bdb, NULL, flags);
+  if(ret) {
     LIBRDF_DEBUG2("Failed to create BDB context - %d\n", ret);
     return 1;
   }
@@ -185,22 +186,23 @@ librdf_hash_bdb_open(void* context, const char *identifier,
    * int DB->open(DB *db, const char *file, const char *database,
    *              DBTYPE type, u_int32_t flags, int mode);
    */
-  flags=is_writable ? DB_CREATE : DB_RDONLY;
+  flags = is_writable ? DB_CREATE : DB_RDONLY;
   if(is_new)
     flags |= DB_TRUNCATE;
 #endif
 
 #if defined(HAVE_BDB_OPEN_6_ARGS) || defined(HAVE_BDB_OPEN_7_ARGS)
 
-#ifdef HAVE_BDB_OPEN_6_ARGS  
+#if defined(HAVE_BDB_OPEN_6_ARGS) && !defined(HAVE_BDB_OPEN_7_ARGS)
 /* 
  * int DB->open(DB *db, const char *file,
  *              const char *database, DBTYPE type, u_int32_t flags, int mode);
  */
-  if((ret=bdb->open(bdb, file, NULL, DB_BTREE, flags, mode))) {
+  ret = bdb->open(bdb, file, NULL, DB_BTREE, flags, mode);
+  if(ret) {
     librdf_log(bdb_context->hash->world, 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
                "BDB V4.0+ open of '%s' failed - %s", file, db_strerror(ret));
-    LIBRDF_FREE(cstring, file);
+    LIBRDF_FREE(char*, file);
     return 1;
   }
 #else
@@ -210,10 +212,11 @@ librdf_hash_bdb_open(void* context, const char *identifier,
  * int DB->open(DB *db, DB_TXN *txnid, const char *file,
  *              const char *database, DBTYPE type, u_int32_t flags, int mode);
  */
-  if((ret=bdb->open(bdb, NULL, file, NULL, DB_BTREE, flags, mode))) {
+  ret = bdb->open(bdb, NULL, file, NULL, DB_BTREE, flags, mode);
+  if(ret) {
     librdf_log(bdb_context->hash->world, 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
                "BDB V4.1+ open of '%s' failed - %s", file, db_strerror(ret));
-    LIBRDF_FREE(cstring, file);
+    LIBRDF_FREE(char*, file);
     return 1;
   }
 #endif
@@ -228,14 +231,15 @@ librdf_hash_bdb_open(void* context, const char *identifier,
   memset(&bdb_info, 0, sizeof(DB_INFO));
   bdb_info.flags=DB_DUP;
  
-  flags=is_writable ? DB_CREATE : DB_RDONLY;
+  flags = is_writable ? DB_CREATE : DB_RDONLY;
   if(is_new)
     flags |= DB_TRUNCATE;
 
-  if((ret=db_open(file, DB_BTREE, flags, mode, NULL, &bdb_info, &bdb))) {
+  ret = db_open(file, DB_BTREE, flags, mode, NULL, &bdb_info, &bdb);
+  if(ret) {
     librdf_log(bdb_context->hash->world, 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
                "BDB V2 open of '%s' failed - %d", file, ret);
-    LIBRDF_FREE(cstring, file);
+    LIBRDF_FREE(char*, file);
     return 1;
   }
 #else
@@ -243,18 +247,19 @@ librdf_hash_bdb_open(void* context, const char *identifier,
   /* V1 prototype:
     const char *file, int flags, int mode, DBTYPE, const void *openinfo
   */
-  flags=is_writable ? O_RDWR|O_CREAT : O_RDONLY
+  flags = is_writable ? O_RDWR | O_CREAT : O_RDONLY
 
-  flags|=R_DUP;
+  flags |= R_DUP;
 
   /* There does not seem to be a V1 flag for truncate */ 
   if(is_new)
     remove(file);
 
-  if((bdb=dbopen(file, flags, mode, DB_BTREE, NULL)) == 0) {
+  bdb = dbopen(file, flags, mode, DB_BTREE, NULL);
+  if(!bdb) {
     librdf_log(bdb_context->hash->world, 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
                "BDB V1 open of '%s' failed - %d", file, ret);
-    LIBRDF_FREE(cstring, file);
+    LIBRDF_FREE(char*, file);
     return 1;
   }
   ret=0;
@@ -299,7 +304,7 @@ librdf_hash_bdb_close(void* context)
   /* V1 */
   ret=db->close(db);
 #endif
-  LIBRDF_FREE(cstring, bdb_context->file_name);
+  LIBRDF_FREE(char*, bdb_context->file_name);
   return ret;
 }
 
@@ -467,7 +472,7 @@ librdf_hash_bdb_cursor_get(void* context,
 
   /* Always initialise BDB version of key */
   bdb_key.data = (char*)key->data;
-  bdb_key.size = key->size;
+  bdb_key.size = LIBRDF_BAD_CAST(u_int32_t, key->size);
   
 #ifdef DB_DBT_MALLOC
   /* BDB V2 or later? */
@@ -579,28 +584,30 @@ librdf_hash_bdb_cursor_get(void* context,
 
   /* Free previous key and values */
   if(cursor->last_key) {
-    LIBRDF_FREE(cstring, cursor->last_key);
+    LIBRDF_FREE(char*, cursor->last_key);
     cursor->last_key=NULL;
   }
     
   if(cursor->last_value) {
-    LIBRDF_FREE(cstring, cursor->last_value);
+    LIBRDF_FREE(char*, cursor->last_value);
     cursor->last_value=NULL;
   }
 
   
 
   if(ret) {
+#ifdef LIBRDF_DEBUG
 #ifdef DB_NOTFOUND
     /* V2 and V3 */
     if(ret != DB_NOTFOUND)
       LIBRDF_DEBUG2("BDB cursor error - %d\n", ret);
+#endif
 #endif
     key->data=NULL;
     return ret;
   }
   
-  cursor->last_key = key->data = LIBRDF_MALLOC(cstring, bdb_key.size);
+  cursor->last_key = key->data = LIBRDF_MALLOC(void*, bdb_key.size);
   if(!key->data) {
     /* always allocated by BDB using system malloc */
     if(flags != LIBRDF_HASH_CURSOR_SET)
@@ -613,7 +620,7 @@ librdf_hash_bdb_cursor_get(void* context,
   key->size = bdb_key.size;
 
   if(value) {
-    cursor->last_value = value->data = LIBRDF_MALLOC(cstring, bdb_value.size);
+    cursor->last_value = value->data = LIBRDF_MALLOC(void*, bdb_value.size);
     if(!value->data) {
       /* always allocated by BDB using system malloc */
       if(flags != LIBRDF_HASH_CURSOR_SET)
@@ -653,10 +660,10 @@ librdf_hash_bdb_cursor_finish(void* context)
     cursor->cursor->c_close(cursor->cursor);
 #endif
   if(cursor->last_key)
-    LIBRDF_FREE(cstring, cursor->last_key);
+    LIBRDF_FREE(char*, cursor->last_key);
     
   if(cursor->last_value)
-    LIBRDF_FREE(cstring, cursor->last_value);
+    LIBRDF_FREE(char*, cursor->last_value);
 }
 
 
@@ -679,6 +686,7 @@ librdf_hash_bdb_put(void* context, librdf_hash_datum *key,
   DBT bdb_value;
   DBT bdb_key;
   int ret;
+  u_int32_t flags = 0;
 
   /* docs say you must zero DBT's before use */
   memset(&bdb_value, 0, sizeof(DBT));
@@ -686,23 +694,25 @@ librdf_hash_bdb_put(void* context, librdf_hash_datum *key,
   
   /* Initialise BDB version of key */
   bdb_key.data = (char*)key->data;
-  bdb_key.size = key->size;
+  bdb_key.size = LIBRDF_BAD_CAST(u_int32_t, key->size);
   
   /* Initialise BDB version of data */
   bdb_value.data = (char*)value->data;
-  bdb_value.size = value->size;
+  bdb_value.size = LIBRDF_BAD_CAST(u_int32_t, value->size);
   
 #ifdef HAVE_BDB_DB_TXN
   /* V2/V3 prototype:
    * int DB->put(DB *db, DB_TXN *txnid, DBT *key, DBT *data, u_int32_t flags); 
    */
-  ret=db->put(db, NULL, &bdb_key, &bdb_value, 0);
+  ret = db->put(db, NULL, &bdb_key, &bdb_value, flags);
 #else
   /* V1 */
-  ret=db->put(db, &bdb_key, &bdb_value, 0);
+  ret = db->put(db, &bdb_key, &bdb_value, flags);
 #endif
+#ifdef LIBRDF_DEBUG
   if(ret)
     LIBRDF_DEBUG2("BDB put failed - %d\n", ret);
+#endif
 
   return (ret != 0);
 }
@@ -730,6 +740,7 @@ librdf_hash_bdb_exists(void* context, librdf_hash_datum *key,
   DBT bdb_key;
   DBT bdb_value;
   int ret;
+  u_int32_t flags = 0;
   
   /* docs say you must zero DBT's before use */
   memset(&bdb_key, 0, sizeof(DBT));
@@ -737,17 +748,19 @@ librdf_hash_bdb_exists(void* context, librdf_hash_datum *key,
   
   /* Initialise BDB version of key */
   bdb_key.data = (char*)key->data;
-  bdb_key.size = key->size;
+  bdb_key.size = LIBRDF_BAD_CAST(u_int32_t, key->size);
 
   if(value) {
     bdb_value.data = (char*)value->data;
-    bdb_value.size = value->size;
+    bdb_value.size = LIBRDF_BAD_CAST(u_int32_t, value->size);
   }
 	
 #ifdef HAVE_BDB_DB_TXN
 #ifdef DB_GET_BOTH
   /* later V2 (sigh)/V3 */
-  ret=db->get(db, NULL, &bdb_key, &bdb_value, (value ? DB_GET_BOTH : 0));
+  if(value)
+    flags = DB_GET_BOTH;
+  ret = db->get(db, NULL, &bdb_key, &bdb_value, flags);
   if(ret == DB_NOTFOUND)
     ret= 0;
   else if(ret) /* failed */
@@ -758,7 +771,7 @@ librdf_hash_bdb_exists(void* context, librdf_hash_datum *key,
   /* earlier V2 */
   if(!value) {
     /* don't care about value, can use standard get */
-    ret=db->get(db, NULL, &bdb_key, &bdb_value, 0);
+    ret = db->get(db, NULL, &bdb_key, &bdb_value, flags);
     if(ret == DB_NOTFOUND)
       ret= 0;
     else if(ret) /* failed */
@@ -868,22 +881,25 @@ librdf_hash_bdb_delete_key(void* context, librdf_hash_datum *key)
   DB* bdb=bdb_context->db;
   DBT bdb_key;
   int ret;
+  u_int32_t flags = 0;
 
   memset(&bdb_key, 0, sizeof(DBT));
 
   /* Initialise BDB version of key */
   bdb_key.data = (char*)key->data;
-  bdb_key.size = key->size;
+  bdb_key.size = LIBRDF_BAD_CAST(u_int32_t, key->size);
   
 #ifdef HAVE_BDB_DB_TXN
   /* V2/V3 */
-  ret=bdb->del(bdb, NULL, &bdb_key, 0);
+  ret = bdb->del(bdb, NULL, &bdb_key, flags);
 #else
   /* V1 */
-  ret=bdb->del(bdb, &bdb_key, 0);
+  ret = bdb->del(bdb, &bdb_key, flags);
 #endif
+#ifdef LIBRDF_DEBUG
   if(ret)
     LIBRDF_DEBUG2("BDB del failed - %d\n", ret);
+#endif
 
   return (ret != 0);
 }
@@ -907,6 +923,7 @@ librdf_hash_bdb_delete_key_value(void* context,
   DB* bdb=bdb_context->db;
   DBT bdb_key, bdb_value;
   int ret;
+  u_int32_t flags = 0;
 #ifdef HAVE_BDB_CURSOR
   DBC* dbc;
 #endif
@@ -916,17 +933,17 @@ librdf_hash_bdb_delete_key_value(void* context,
 
   /* Initialise BDB version of key */
   bdb_key.data = (char*)key->data;
-  bdb_key.size = key->size;
+  bdb_key.size = LIBRDF_BAD_CAST(u_int32_t, key->size);
   
   bdb_value.data = (char*)value->data;
-  bdb_value.size = value->size;
+  bdb_value.size = LIBRDF_BAD_CAST(u_int32_t, value->size);
   
 #ifdef HAVE_BDB_CURSOR
 #ifdef HAVE_BDB_CURSOR_4_ARGS
   /* V3 prototype:
    * int DB->cursor(DB *db, DB_TXN *txnid, DBC **cursorp, u_int32_t flags);
    */
-  if(bdb->cursor(bdb, NULL, &dbc, 0))
+  if(bdb->cursor(bdb, NULL, &dbc, flags))
     return 1;
 #else
   /* V2 prototype:
@@ -941,11 +958,11 @@ librdf_hash_bdb_delete_key_value(void* context,
    */
 #ifdef DB_GET_BOTH
   /* later V2 (sigh) / V3 */
-  ret=dbc->c_get(dbc, &bdb_key, &bdb_value, DB_GET_BOTH);
+  flags = DB_GET_BOTH;
 #else
   /* earlier V2 probably gives a memory leak */
-  ret=dbc->c_get(dbc, &bdb_key, &bdb_value, 0);
 #endif
+  ret = dbc->c_get(dbc, &bdb_key, &bdb_value, flags);
   if(ret) {
     dbc->c_close(dbc);
     return 1;
@@ -969,8 +986,10 @@ librdf_hash_bdb_delete_key_value(void* context,
   ret=bdb->del(bdb, &bdb_key, R_CURSOR);
 #endif
 
+#ifdef LIBRDF_DEBUG
   if(ret)
     LIBRDF_DEBUG2("BDB del failed - %d\n", ret);
+#endif
 
   return (ret != 0);
 }
