@@ -1,25 +1,25 @@
 /* -*- Mode: c; c-basic-offset: 2 -*-
  *
- * rdf_parser.c - RDF Parser using Raptor
+ * rdf_parser_raptor.c - RDF Parser using Raptor
  *
- * Copyright (C) 2000-2008, David Beckett http://www.dajobe.org/
+ * Copyright (C) 2000-2011, David Beckett http://www.dajobe.org/
  * Copyright (C) 2000-2004, University of Bristol, UK http://www.bristol.ac.uk/
- * 
+ *
  * This package is Free Software and part of Redland http://librdf.org/
- * 
+ *
  * It is licensed under the following three licenses as alternatives:
  *   1. GNU Lesser General Public License (LGPL) V2.1 or any newer version
  *   2. GNU General Public License (GPL) V2 or any newer version
  *   3. Apache License, V2.0 or any newer version
- * 
+ *
  * You may not use this file except in compliance with at least one of
  * the above three licenses.
- * 
+ *
  * See LICENSE.html or LICENSE.txt at the top of this package for the
  * complete terms and further detail along with the license texts for
  * the licenses in COPYING.LIB, COPYING and LICENSE-2.0.txt respectively.
- * 
- * 
+ *
+ *
  */
 
 
@@ -52,13 +52,12 @@ static void librdf_parser_raptor_serialise_finished(void* context);
 
 typedef struct {
   librdf_parser *parser;        /* librdf parser object */
-  librdf_hash *bnode_hash;      /* bnode id (raptor=>internal) map */
   raptor_parser *rdf_parser;    /* source URI string (for raptor) */
   const char *parser_name;      /* raptor parser name to use */
 
   raptor_sequence* nspace_prefixes;
   raptor_sequence* nspace_uris;
-  
+
   int errors;
   int warnings;
 
@@ -77,10 +76,10 @@ typedef struct {
 
   /* when finished */
   int finished;
-  
+
   /* when storing into a model - librdf_parser_raptor_parse_uri_into_model */
   librdf_model *model;
-  
+
   /* The set of statements pending is a sequence, with 'current'
    * as the first entry and any remaining ones held in 'statements'.
    * The latter are filled by the parser
@@ -92,7 +91,7 @@ typedef struct {
 
 
 static int
-librdf_parser_raptor_relay_filter(void* user_data, raptor_uri* uri) 
+librdf_parser_raptor_relay_filter(void* user_data, raptor_uri* uri)
 {
   librdf_parser* parser=(librdf_parser*)user_data;
   return parser->uri_filter(parser->uri_filter_user_data, (librdf_uri*)uri);
@@ -105,11 +104,11 @@ librdf_parser_raptor_relay_filter(void* user_data, raptor_uri* uri)
  * @context: context
  *
  * Initialise the raptor RDF parser.
- * 
+ *
  * Return value: non 0 on failure
  **/
 static int
-librdf_parser_raptor_init(librdf_parser *parser, void *context) 
+librdf_parser_raptor_init(librdf_parser *parser, void *context)
 {
   librdf_parser_raptor_context* pcontext=(librdf_parser_raptor_context*)context;
 
@@ -119,12 +118,7 @@ librdf_parser_raptor_init(librdf_parser *parser, void *context)
    * from when there was just one parser
    */
   if(!strcmp(pcontext->parser_name, "raptor"))
-    pcontext->parser_name="rdfxml";
-
-  /* New in-memory hash for mapping bnode IDs */
-  pcontext->bnode_hash=librdf_new_hash(parser->world, NULL);
-  if(!pcontext->bnode_hash)
-    return 1;
+    pcontext->parser_name = "rdfxml";
 
   pcontext->rdf_parser = raptor_new_parser(parser->world->raptor_world_ptr,
                                            pcontext->parser_name);
@@ -132,9 +126,11 @@ librdf_parser_raptor_init(librdf_parser *parser, void *context)
   if(!pcontext->rdf_parser)
     return 1;
 
+  librdf_raptor_reset_bnode_hash(parser->world);
+
   return 0;
 }
-  
+
 
 /**
  * librdf_parser_raptor_terminate:
@@ -144,34 +140,33 @@ librdf_parser_raptor_init(librdf_parser *parser, void *context)
  *
  **/
 static void
-librdf_parser_raptor_terminate(void *context) 
+librdf_parser_raptor_terminate(void *context)
 {
   librdf_parser_raptor_context* pcontext=(librdf_parser_raptor_context*)context;
 
+  librdf_raptor_free_bnode_hash(pcontext->parser->world);
+  
   if(pcontext->stream_context)
     librdf_parser_raptor_serialise_finished(pcontext->stream_context);
 
   if(pcontext->www)
     raptor_free_www(pcontext->www);
-  
+
   if(pcontext->rdf_parser)
     raptor_free_parser(pcontext->rdf_parser);
 
-  if(pcontext->bnode_hash)
-    librdf_free_hash(pcontext->bnode_hash);
-
   if(pcontext->nspace_prefixes)
     raptor_free_sequence(pcontext->nspace_prefixes);
   if(pcontext->nspace_uris)
     raptor_free_sequence(pcontext->nspace_uris);
 }
-  
+
 
 /*
  * librdf_parser_raptor_new_statement_handler - helper callback function for raptor RDF when a new triple is asserted
  * @context: context for callback
  * @statement: raptor_statement
- * 
+ *
  * Adds the statement to the list of statements.
  */
 static void
@@ -208,9 +203,9 @@ librdf_parser_raptor_new_statement_handler(void *context,
     librdf_free_statement(statement);
     return;
   }
-  
+
   librdf_statement_set_subject(statement, node);
-  
+
 
   if(rstatement->predicate->type == RAPTOR_TERM_TYPE_URI) {
     node = librdf_new_node_from_uri(world, (librdf_uri*)rstatement->predicate->value.uri);
@@ -261,15 +256,13 @@ librdf_parser_raptor_new_statement_handler(void *context,
 
   librdf_statement_set_object(statement, node);
 
-#ifdef LIBRDF_DEBUG
-#if LIBRDF_DEBUG > 1
+#if defined(LIBRDF_DEBUG) && LIBRDF_DEBUG > 1
   if(1) {
     raptor_iostream *iostr;
     iostr = raptor_new_iostream_to_file_handle(world->raptor_world_ptr, stderr);
     librdf_statement_write(statement, iostr);
     raptor_free_iostream(iostr);
   }
-#endif
 #endif
 
   if(scontext->model) {
@@ -292,11 +285,11 @@ librdf_parser_raptor_new_statement_handler(void *context,
  * librdf_parser_raptor_namespace_handler - helper callback function for raptor RDF when a namespace is seen
  * @context: context for callback
  * @statement: raptor_statement
- * 
+ *
  * Adds the statement to the list of statements.
  */
 static void
-librdf_parser_raptor_namespace_handler(void* user_data, 
+librdf_parser_raptor_namespace_handler(void* user_data,
                                        raptor_namespace *nspace)
 {
   librdf_parser_raptor_context* pcontext=(librdf_parser_raptor_context*)user_data;
@@ -305,24 +298,24 @@ librdf_parser_raptor_namespace_handler(void* user_data,
   size_t prefix_length;
   librdf_uri* uri;
   int i;
-  
+
   uri=(librdf_uri*)raptor_namespace_get_uri(nspace);
   if(!uri)
     return;
-  
+
   for(i=0; i < raptor_sequence_size(pcontext->nspace_uris); i++) {
     librdf_uri* u=(librdf_uri*)raptor_sequence_get_at(pcontext->nspace_uris, i);
     if(librdf_uri_equals(uri, u))
       return;
   }
-  
+
   /* new namespace */
   uri=librdf_new_uri_from_uri(uri);
   raptor_sequence_push(pcontext->nspace_uris, uri);
 
   prefix=raptor_namespace_get_counted_prefix(nspace, &prefix_length);
   if(prefix) {
-    nprefix=(unsigned char*)LIBRDF_MALLOC(cstring, prefix_length+1);
+    nprefix = LIBRDF_MALLOC(unsigned char*, prefix_length + 1);
     /* FIXME: what if nprefix alloc failed? now just pushes NULL to sequence */
     if(nprefix)
       strncpy((char*)nprefix, (const char*)prefix, prefix_length+1);
@@ -339,20 +332,21 @@ librdf_parser_raptor_namespace_handler(void* user_data,
 /*
  * librdf_parser_raptor_get_next_statement - helper function to get the next statement
  * @context: serialisation context
- * 
+ *
  * Return value: >0 if a statement found, 0 at end of file, or <0 on error
  */
 static int
 librdf_parser_raptor_get_next_statement(librdf_parser_raptor_stream_context *context) {
   unsigned char buffer[RAPTOR_IO_BUFFER_LEN];
   int status=0;
-  
+
   if(context->finished || !context->fh)
     return 0;
-  
+
   context->current=NULL;
   while(!feof(context->fh)) {
-    int len, ret;
+    size_t len;
+    int ret;
 
     len = fread(buffer, 1, RAPTOR_IO_BUFFER_LEN, context->fh);
     ret = raptor_parser_parse_chunk(context->pcontext->rdf_parser, buffer, len,
@@ -376,35 +370,11 @@ librdf_parser_raptor_get_next_statement(librdf_parser_raptor_stream_context *con
 
   if(feof(context->fh) || status <1)
     context->finished=1;
-  
+
   return status;
 }
 
 
-static unsigned char*
-librdf_parser_raptor_generate_id_handler(void *user_data,
-                                         unsigned char *user_bnodeid) 
-{
-  librdf_parser_raptor_context* pcontext=(librdf_parser_raptor_context*)user_data;
-  if(user_bnodeid) {
-    unsigned char *mapped_id=(unsigned char*)librdf_hash_get(pcontext->bnode_hash, (const char*)user_bnodeid);
-    if(!mapped_id) {
-      mapped_id=librdf_world_get_genid(pcontext->parser->world);
-      if(mapped_id && librdf_hash_put_strings(pcontext->bnode_hash, (char*)user_bnodeid, (char*)mapped_id)) {
-        /* error -> free mapped_id and return NULL */
-        LIBRDF_FREE(cstring, mapped_id);
-        mapped_id=NULL;
-      }
-    }
-    /* always free passed in bnodeid */
-    raptor_free_memory(user_bnodeid);
-    return mapped_id;
-  }
-  else
-    return librdf_world_get_genid(pcontext->parser->world);
-}
-
-
 /*
  * librdf_parser_raptor_parse_file_handle_as_stream:
  * @context: parser context
@@ -442,16 +412,17 @@ librdf_parser_raptor_parse_file_handle_as_stream(void *context,
   if(need_base_uri && !base_uri) {
     librdf_log(pcontext->parser->world,
                0, LIBRDF_LOG_ERROR, LIBRDF_FROM_PARSER, NULL,
-               "Missing base URI for %s parser", 
+               "Missing base URI for %s parser",
                pcontext->parser_name);
     return NULL;
   }
-  
+
 
   pcontext->errors=0;
   pcontext->warnings=0;
-  
-  scontext=(librdf_parser_raptor_stream_context*)LIBRDF_CALLOC(librdf_parser_raptor_stream_context, 1, sizeof(librdf_parser_raptor_stream_context));
+
+  scontext = LIBRDF_CALLOC(librdf_parser_raptor_stream_context*, 1,
+                           sizeof(*scontext));
   if(!scontext)
     goto oom;
 
@@ -474,18 +445,13 @@ librdf_parser_raptor_parse_file_handle_as_stream(void *context,
   pcontext->nspace_uris = raptor_new_sequence((raptor_data_free_handler)librdf_free_uri, NULL);
   if(!pcontext->nspace_uris)
     goto oom;
-  
+
   raptor_parser_set_statement_handler(pcontext->rdf_parser, scontext,
                                       librdf_parser_raptor_new_statement_handler);
   raptor_parser_set_namespace_handler(pcontext->rdf_parser, pcontext,
                                       librdf_parser_raptor_namespace_handler);
 
-  raptor_world_set_generate_bnodeid_handler(pcontext->parser->world->raptor_world_ptr,
-                                            pcontext,
-                                            librdf_parser_raptor_generate_id_handler);
 
-  
-  
   scontext->fh=fh;
   scontext->close_fh=close_fh;
 
@@ -501,7 +467,7 @@ librdf_parser_raptor_parse_file_handle_as_stream(void *context,
   if(!rc) {
     /* start parsing; initialises scontext->statements, scontext->current */
     librdf_parser_raptor_get_next_statement(scontext);
-    
+
     stream=librdf_new_stream(pcontext->parser->world,
                              (void*)scontext,
                              &librdf_parser_raptor_serialise_end_of_stream,
@@ -532,7 +498,7 @@ librdf_parser_raptor_parse_uri_as_stream_write_bytes_handler(raptor_www *www,
                                                              size_t nmemb)
 {
   librdf_parser_raptor_stream_context* scontext = (librdf_parser_raptor_stream_context*)userdata;
-  int len = size*nmemb;
+  size_t len = size * nmemb;
   int rc;
 
   rc = raptor_parser_parse_chunk(scontext->pcontext->rdf_parser,
@@ -549,17 +515,19 @@ librdf_parser_raptor_parse_uri_as_stream_write_bytes_handler(raptor_www *www,
  * @uri: #librdf_uri URI of RDF content source
  * @string: or content string
  * @length: length of the string or 0 if not yet counted
+ * @iostream: an iostream to parse
  * @base_uri: #librdf_uri URI of the content location or NULL if the same
  *
  * Retrieve the content at URI/string and parse it into a #librdf_stream.
  *
- * Only one of uri or string may be given
+ * Precisely one of uri, string, and iostream must be non-NULL
  *
  **/
 static librdf_stream*
 librdf_parser_raptor_parse_as_stream_common(void *context, librdf_uri *uri,
                                             const unsigned char *string,
                                             size_t length,
+                                            raptor_iostream *iostream,
                                             librdf_uri *base_uri)
 {
   librdf_parser_raptor_context* pcontext=(librdf_parser_raptor_context*)context;
@@ -571,7 +539,7 @@ librdf_parser_raptor_parse_as_stream_common(void *context, librdf_uri *uri,
 
   if(!base_uri && uri)
     base_uri=uri;
-  
+
   desc = raptor_parser_get_description(pcontext->rdf_parser);
   if(!desc) {
     librdf_log(pcontext->parser->world,
@@ -585,23 +553,23 @@ librdf_parser_raptor_parse_as_stream_common(void *context, librdf_uri *uri,
   if(need_base_uri && !base_uri) {
     librdf_log(pcontext->parser->world,
                0, LIBRDF_LOG_ERROR, LIBRDF_FROM_PARSER, NULL,
-               "Missing base URI for %s parser", 
+               "Missing base URI for %s parser",
                pcontext->parser_name);
-    
+
     return NULL;
   }
-  
+
 
   pcontext->errors=0;
   pcontext->warnings=0;
-  
+
   if(uri && librdf_uri_is_file_uri(uri)) {
     char* filename=(char*)librdf_uri_to_filename(uri);
     FILE *fh;
 
     if(!filename)
       return NULL;
-    
+
     fh=fopen(filename, "r");
     if(!fh) {
       librdf_log(pcontext->parser->world, 0, LIBRDF_LOG_ERROR,
@@ -618,9 +586,10 @@ librdf_parser_raptor_parse_as_stream_common(void *context, librdf_uri *uri,
     SYSTEM_FREE(filename);
     return stream;
   }
-  
 
-  scontext=(librdf_parser_raptor_stream_context*)LIBRDF_CALLOC(librdf_parser_raptor_stream_context, 1, sizeof(librdf_parser_raptor_stream_context));
+
+  scontext = LIBRDF_CALLOC(librdf_parser_raptor_stream_context*, 1,
+                           sizeof(*scontext));
   if(!scontext)
     goto oom;
 
@@ -643,16 +612,12 @@ librdf_parser_raptor_parse_as_stream_common(void *context, librdf_uri *uri,
   pcontext->nspace_uris = raptor_new_sequence((raptor_data_free_handler)librdf_free_uri, NULL);
   if(!pcontext->nspace_uris)
     goto oom;
-  
+
   raptor_parser_set_statement_handler(pcontext->rdf_parser, scontext,
                                       librdf_parser_raptor_new_statement_handler);
   raptor_parser_set_namespace_handler(pcontext->rdf_parser, pcontext,
                                       librdf_parser_raptor_namespace_handler);
 
-  raptor_world_set_generate_bnodeid_handler(pcontext->parser->world->raptor_world_ptr,
-                                            pcontext,
-                                            librdf_parser_raptor_generate_id_handler);
-
   if(pcontext->parser->uri_filter)
     raptor_parser_set_uri_filter(pcontext->rdf_parser,
                                  librdf_parser_raptor_relay_filter,
@@ -667,14 +632,14 @@ librdf_parser_raptor_parse_as_stream_common(void *context, librdf_uri *uri,
     pcontext->www = raptor_new_www(pcontext->parser->world->raptor_world_ptr);
     if(!pcontext->www)
       goto oom;
-    
+
     accept_h=raptor_parser_get_accept_header(pcontext->rdf_parser);
     if(accept_h) {
       raptor_www_set_http_accept(pcontext->www, accept_h);
-      raptor_free_memory((char*)accept_h);
+      raptor_free_memory((void*)accept_h);
     }
 
-    raptor_www_set_write_bytes_handler(pcontext->www, 
+    raptor_www_set_write_bytes_handler(pcontext->www,
                                        librdf_parser_raptor_parse_uri_as_stream_write_bytes_handler,
                                        scontext);
 
@@ -686,27 +651,45 @@ librdf_parser_raptor_parse_as_stream_common(void *context, librdf_uri *uri,
       librdf_parser_raptor_serialise_finished((void*)scontext);
       return NULL;
     }
-    
+
     raptor_www_fetch(pcontext->www, (raptor_uri*)uri);
     raptor_parser_parse_chunk(pcontext->rdf_parser, NULL, 0, 1);
 
     raptor_free_www(pcontext->www);
 
     pcontext->www = NULL;
-  } else {
+  } else if (string) {
     status = raptor_parser_parse_start(pcontext->rdf_parser,
                                        (raptor_uri*)base_uri);
     if(status) {
       librdf_parser_raptor_serialise_finished((void*)scontext);
       return NULL;
     }
-    
+
     if(!length)
       length = strlen((const char*)string);
 
     raptor_parser_parse_chunk(pcontext->rdf_parser, string, length, 1);
+  } else if (iostream) {
+    status = raptor_parser_parse_start(pcontext->rdf_parser, (raptor_uri*)base_uri);
+    if(status) {
+      librdf_parser_raptor_serialise_finished((void*)scontext);
+      return NULL;
+    }
+
+    status = raptor_parser_parse_iostream(pcontext->rdf_parser,
+                                          iostream,
+                                          (raptor_uri*)base_uri);
+    if(status) {
+      librdf_parser_raptor_serialise_finished((void*)scontext);
+      return NULL;
+    }
+  } else {
+    /* All three of URI, string and iostream are null.  That's a coding error. */
+    librdf_parser_raptor_serialise_finished((void*)scontext);
+    return NULL;
   }
-  
+
 
   /* get first statement, else is empty */
   scontext->current=(librdf_statement*)librdf_list_pop(scontext->statements);
@@ -748,6 +731,7 @@ librdf_parser_raptor_parse_uri_as_stream(void *context, librdf_uri *uri,
 {
   return librdf_parser_raptor_parse_as_stream_common(context, uri,
                                                      NULL, 0,
+                                                     0,
                                                      base_uri);
 }
 
@@ -763,12 +747,13 @@ librdf_parser_raptor_parse_uri_as_stream(void *context, librdf_uri *uri,
  *
  **/
 static librdf_stream*
-librdf_parser_raptor_parse_string_as_stream(void *context, 
+librdf_parser_raptor_parse_string_as_stream(void *context,
                                             const unsigned char *string,
                                             librdf_uri *base_uri)
 {
   return librdf_parser_raptor_parse_as_stream_common(context, NULL,
                                                      string, 0,
+                                                     0,
                                                      base_uri);
 }
 
@@ -781,33 +766,59 @@ librdf_parser_raptor_parse_string_as_stream(void *context,
  * @base_uri: the base URI to use
  *
  * Parse a counted string of content to a librdf_stream of statements.
- * 
+ *
  * Return value: #librdf_stream of statements or NULL
  **/
 static librdf_stream*
 librdf_parser_raptor_parse_counted_string_as_stream(void *context,
                                                     const unsigned char *string,
                                                     size_t length,
-                                                    librdf_uri* base_uri) 
+                                                    librdf_uri* base_uri)
 {
   return librdf_parser_raptor_parse_as_stream_common(context, NULL,
                                                      string, length,
+                                                     0,
                                                      base_uri);
 }
 
+/**
+ * librdf_parser_raptor_parse_iostream_as_stream:
+ * @context: parser context
+ * @iostream: iostream content to parse
+ * @base_uri: #librdf_uri URI of the content location or NULL if the same
+ *
+ * Parse the content in an iostream and return a librdf_stream.
+ *
+ *
+ **/
+static librdf_stream*
+librdf_parser_raptor_parse_iostream_as_stream(void *context,
+                                              raptor_iostream *iostream,
+                                              librdf_uri *base_uri)
+{
+  return librdf_parser_raptor_parse_as_stream_common(context, NULL,
+                                                     0, 0,
+                                                     iostream,
+                                                     base_uri);
+}
+
+
 /*
  * librdf_parser_raptor_parse_into_model_common:
  * @context: parser context
- * @uri: #librdf_uri URI of RDF content source or NULL if from a string or a file handler
- * @string: string content to parser or NULL if from a URI or a file handler
- * @fh: FILE* content source or NULL if from a URI or a string
+ * @uri: #librdf_uri URI of RDF content source, or NULL
+ * @string: string content to parser, or NULL
  * @length: length of the string or 0 if not yet counted
+ * @fh: FILE* content source, or NULL
+ * @iostream: iostream content, or NULL
  * @base_uri: #librdf_uri URI of the content location or NULL
  * @model: #librdf_model of model
  *
  * Retrieve the RDF content at URI and store it into a librdf_model.
  *
- * Parses the content at @uri, @string or @fh and store it in the given model.
+ * Precisely one of uri, string, fh or iostream must be non-null.
+ *
+ * Parses the content at @uri, @string, @fh or @iostream and store it in the given model.
  *
  * Return value: non 0 on failure
  **/
@@ -815,8 +826,9 @@ static int
 librdf_parser_raptor_parse_into_model_common(void *context,
                                              librdf_uri *uri,
                                              const unsigned char *string,
-                                             FILE *fh,
                                              size_t length,
+                                             FILE *fh,
+                                             raptor_iostream *iostream,
                                              librdf_uri *base_uri,
                                              librdf_model* model)
 {
@@ -838,11 +850,11 @@ librdf_parser_raptor_parse_into_model_common(void *context,
     return -1;
   }
   need_base_uri = desc->flags & RAPTOR_SYNTAX_NEED_BASE_URI;
-  
+
   if(need_base_uri && !base_uri) {
     librdf_log(pcontext->parser->world,
                0, LIBRDF_LOG_ERROR, LIBRDF_FROM_PARSER, NULL,
-               "Missing base URI for %s parser", 
+               "Missing base URI for %s parser",
                pcontext->parser_name);
 
     return 1;
@@ -850,8 +862,9 @@ librdf_parser_raptor_parse_into_model_common(void *context,
 
   pcontext->errors=0;
   pcontext->warnings=0;
-  
-  scontext=(librdf_parser_raptor_stream_context*)LIBRDF_CALLOC(librdf_parser_raptor_stream_context, 1, sizeof(librdf_parser_raptor_stream_context));
+
+  scontext = LIBRDF_CALLOC(librdf_parser_raptor_stream_context*, 1,
+                           sizeof(*scontext));
   if(!scontext)
     goto oom;
 
@@ -876,10 +889,6 @@ librdf_parser_raptor_parse_into_model_common(void *context,
   raptor_parser_set_namespace_handler(pcontext->rdf_parser, pcontext,
                                       librdf_parser_raptor_namespace_handler);
 
-  raptor_world_set_generate_bnodeid_handler(pcontext->parser->world->raptor_world_ptr,
-                                            pcontext,
-                                            librdf_parser_raptor_generate_id_handler);
-
   /* direct into model */
   scontext->model=model;
 
@@ -887,7 +896,7 @@ librdf_parser_raptor_parse_into_model_common(void *context,
     raptor_parser_set_uri_filter(pcontext->rdf_parser,
                                  librdf_parser_raptor_relay_filter,
                                  pcontext->parser);
-    
+
   if(uri) {
     status = raptor_parser_parse_uri(pcontext->rdf_parser, (raptor_uri*)uri,
                                      (raptor_uri*)base_uri);
@@ -898,8 +907,13 @@ librdf_parser_raptor_parse_into_model_common(void *context,
         length = strlen((const char*)string);
       status = raptor_parser_parse_chunk(pcontext->rdf_parser, string, length, 1);
     }
-  } else {
+  } else if(fh) {
     status = raptor_parser_parse_file_stream(pcontext->rdf_parser, fh, NULL, (raptor_uri*)base_uri);
+  } else if(iostream) {
+    status = raptor_parser_parse_iostream(pcontext->rdf_parser, iostream,  (raptor_uri*)base_uri);
+  } else {
+    /* All four of URI, string, fh and iostream are null.  That's a coding error. */
+    status = -1;
   }
 
   librdf_parser_raptor_serialise_finished((void*)scontext);
@@ -930,12 +944,12 @@ librdf_parser_raptor_parse_into_model_common(void *context,
  * Return value: non 0 on failure
  **/
 static int
-librdf_parser_raptor_parse_uri_into_model(void *context, librdf_uri *uri, 
+librdf_parser_raptor_parse_uri_into_model(void *context, librdf_uri *uri,
                                           librdf_uri *base_uri,
                                           librdf_model* model)
 {
-  return librdf_parser_raptor_parse_into_model_common(context, uri, 
-                                                      NULL, NULL, 0,
+  return librdf_parser_raptor_parse_into_model_common(context, uri,
+                                                      NULL, 0, NULL, NULL,
                                                       base_uri, model);}
 
 
@@ -953,13 +967,13 @@ librdf_parser_raptor_parse_uri_into_model(void *context, librdf_uri *uri,
  * Return value: non 0 on failure
  **/
 static int
-librdf_parser_raptor_parse_string_into_model(void *context, 
+librdf_parser_raptor_parse_string_into_model(void *context,
                                              const unsigned char *string,
                                              librdf_uri *base_uri,
                                              librdf_model* model)
 {
   return librdf_parser_raptor_parse_into_model_common(context, NULL,
-                                                      string, NULL, 0,
+                                                      string, 0, NULL, NULL,
                                                       base_uri, model);
 }
 
@@ -984,7 +998,7 @@ librdf_parser_raptor_parse_file_handle_into_model(void *context, FILE *fh,
                                                   librdf_model* model)
 {
   int status=librdf_parser_raptor_parse_into_model_common(context, NULL,
-                                                          NULL, fh, 0,
+                                                          NULL, 0, fh, NULL,
                                                           base_uri, model);
 
   if (close_fh)
@@ -1003,18 +1017,41 @@ librdf_parser_raptor_parse_file_handle_into_model(void *context, FILE *fh,
  * @model: the model to use
  *
  * INTERNAL - Parse a counted string of content into an #librdf_model.
- * 
+ *
  * Return value: non 0 on failure
  **/
 static int
 librdf_parser_raptor_parse_counted_string_into_model(void *context,
                                                      const unsigned char *string,
                                                      size_t length,
-                                                     librdf_uri* base_uri, 
+                                                     librdf_uri* base_uri,
                                                      librdf_model* model)
 {
   return librdf_parser_raptor_parse_into_model_common(context, NULL,
-                                                      string, NULL, length,
+                                                      string, length, NULL, NULL,
+                                                      base_uri, model);
+}
+
+
+/**
+ * librdf_parser_raptor_parse_iostream_into_model:
+ * @context: parser context
+ * @iostream: the content to parse
+ * @base_uri: the base URI to use
+ * @model: the model to use
+ *
+ * INTERNAL - Parse an iostream of content into an #librdf_model.
+ *
+ * Return value: non 0 on failure
+ **/
+static int
+librdf_parser_raptor_parse_iostream_into_model(void *context,
+                                               raptor_iostream *iostream,
+                                               librdf_uri* base_uri,
+                                               librdf_model* model)
+{
+  return librdf_parser_raptor_parse_into_model_common(context, NULL,
+                                                      NULL, 0, NULL, iostream,
                                                       base_uri, model);
 }
 
@@ -1024,7 +1061,7 @@ librdf_parser_raptor_parse_counted_string_into_model(void *context,
  * @context: the context passed in by #librdf_stream
  *
  * Check for the end of the stream of statements from the raptor RDF parser.
- * 
+ *
  * Return value: non 0 at end of stream
  **/
 static int
@@ -1041,7 +1078,7 @@ librdf_parser_raptor_serialise_end_of_stream(void* context)
  * @context: the context passed in by #librdf_stream
  *
  * Move to the next librdf_statement in the stream of statements from the raptor RDF parse.
- * 
+ *
  * Return value: non 0 at end of stream
  **/
 static int
@@ -1057,7 +1094,7 @@ librdf_parser_raptor_serialise_next_statement(void* context)
     scontext->current=(librdf_statement*)librdf_list_pop(scontext->statements);
     if(scontext->current)
       break;
-  
+
     /* else get a new one */
 
     /* 0 is end, <0 is error.  Either way stop */
@@ -1075,7 +1112,7 @@ librdf_parser_raptor_serialise_next_statement(void* context)
  * @flags: the context get method flags
  *
  * Get the current librdf_statement from the stream of statements from the raptor RDF parse.
- * 
+ *
  * Return value: a new #librdf_statement or NULL on error or if no statements found.
  **/
 static void*
@@ -1089,7 +1126,7 @@ librdf_parser_raptor_serialise_get_statement(void* context, int flags)
 
     case LIBRDF_ITERATOR_GET_METHOD_GET_CONTEXT:
       return NULL;
-      
+
     default:
       librdf_log(scontext->pcontext->parser->world,
                  0, LIBRDF_LOG_ERROR, LIBRDF_FROM_PARSER, NULL,
@@ -1114,6 +1151,7 @@ librdf_parser_raptor_serialise_finished(void* context)
 
   if(scontext) {
     librdf_statement* statement;
+    librdf_world* world = scontext->pcontext->parser ? scontext->pcontext->parser->world : NULL;
 
     if(scontext->current)
       librdf_free_statement(scontext->current);
@@ -1125,10 +1163,12 @@ librdf_parser_raptor_serialise_finished(void* context)
     }
 
     if(scontext->fh && scontext->close_fh)
-      fclose(scontext->fh); 
+      fclose(scontext->fh);
 
     if(scontext->pcontext)
-      scontext->pcontext->stream_context=NULL;
+      scontext->pcontext->stream_context = NULL;
+
+    librdf_raptor_reset_bnode_hash(world);
 
     LIBRDF_FREE(librdf_parser_raptor_context, scontext);
   }
@@ -1136,7 +1176,7 @@ librdf_parser_raptor_serialise_finished(void* context)
 
 
 static librdf_node*
-librdf_parser_raptor_get_feature(void* context, librdf_uri *feature) 
+librdf_parser_raptor_get_feature(void* context, librdf_uri *feature)
 {
   librdf_parser_raptor_context* pcontext=(librdf_parser_raptor_context*)context;
   unsigned char intbuffer[20]; /* FIXME */
@@ -1148,7 +1188,7 @@ librdf_parser_raptor_get_feature(void* context, librdf_uri *feature)
   uri_string=librdf_uri_as_string(feature);
   if(!uri_string)
     return NULL;
-  
+
   if(!strcmp((const char*)uri_string, LIBRDF_PARSER_FEATURE_ERROR_COUNT)) {
     sprintf((char*)intbuffer, "%d", pcontext->errors);
     return librdf_new_node_from_typed_literal(pcontext->parser->world,
@@ -1171,19 +1211,19 @@ librdf_parser_raptor_get_feature(void* context, librdf_uri *feature)
                                                 intbuffer, NULL, NULL);
     }
   }
-  
+
   return NULL;
 }
 
 
 static int
-librdf_parser_raptor_set_feature(void* context, 
-                                 librdf_uri *feature, librdf_node *value) 
+librdf_parser_raptor_set_feature(void* context,
+                                 librdf_uri *feature, librdf_node *value)
 {
   librdf_parser_raptor_context* pcontext=(librdf_parser_raptor_context*)context;
   raptor_option feature_i;
   const unsigned char* value_s;
-  
+
   if(!feature)
     return 1;
 
@@ -1191,10 +1231,10 @@ librdf_parser_raptor_set_feature(void* context,
   feature_i = raptor_world_get_option_from_uri(pcontext->parser->world->raptor_world_ptr, (raptor_uri*)feature);
   if((int)feature_i < 0)
     return 1;
-  
+
   if(!librdf_node_is_literal(value))
     return 1;
-  
+
   value_s=(const unsigned char*)librdf_node_get_literal_value(value);
 
   return raptor_parser_set_option(pcontext->rdf_parser, feature_i,
@@ -1203,21 +1243,21 @@ librdf_parser_raptor_set_feature(void* context,
 
 
 static char*
-librdf_parser_raptor_get_accept_header(void* context) 
+librdf_parser_raptor_get_accept_header(void* context)
 {
-  librdf_parser_raptor_context* pcontext=(librdf_parser_raptor_context*)context;
+  librdf_parser_raptor_context* pcontext = (librdf_parser_raptor_context*)context;
   const char* accept;
   char* r_accept;
   size_t length;
-  
-  accept=raptor_parser_get_accept_header(pcontext->rdf_parser);
+
+  accept = raptor_parser_get_accept_header(pcontext->rdf_parser);
   if(!accept)
     return NULL;
-  length=strlen(accept);
-  r_accept=(char*)LIBRDF_MALLOC(cstring, length+1);
+  length = strlen(accept);
+  r_accept = (char *)librdf_alloc_memory(length + 1);
   strncpy(r_accept, accept, length+1);
   raptor_free_memory((void*)accept);
-  
+
   return r_accept;
 }
 
@@ -1272,13 +1312,13 @@ librdf_parser_raptor_get_namespaces_seen_count(void* context)
  * @factory: factory
  *
  * Register the raptor RDF parser with the RDF parser factory.
- * 
+ *
  **/
 static void
-librdf_parser_raptor_register_factory(librdf_parser_factory *factory) 
+librdf_parser_raptor_register_factory(librdf_parser_factory *factory)
 {
   factory->context_length = sizeof(librdf_parser_raptor_context);
-  
+
   factory->init  = librdf_parser_raptor_init;
   factory->terminate  = librdf_parser_raptor_terminate;
   factory->parse_uri_as_stream = librdf_parser_raptor_parse_uri_as_stream;
@@ -1287,6 +1327,8 @@ librdf_parser_raptor_register_factory(librdf_parser_factory *factory)
   factory->parse_string_into_model = librdf_parser_raptor_parse_string_into_model;
   factory->parse_counted_string_as_stream = librdf_parser_raptor_parse_counted_string_as_stream;
   factory->parse_counted_string_into_model = librdf_parser_raptor_parse_counted_string_into_model;
+  factory->parse_iostream_as_stream = librdf_parser_raptor_parse_iostream_as_stream;
+  factory->parse_iostream_into_model = librdf_parser_raptor_parse_iostream_into_model;
   factory->parse_file_handle_as_stream = librdf_parser_raptor_parse_file_handle_as_stream;
   factory->parse_file_handle_into_model = librdf_parser_raptor_parse_file_handle_into_model;
   factory->get_feature = librdf_parser_raptor_get_feature;
@@ -1324,7 +1366,13 @@ librdf_parser_raptor_constructor(librdf_world *world)
       /* reached the end of the parsers, now register the default one */
       i = 0;
       desc = raptor_world_get_parser_description(world->raptor_world_ptr, i);
+      if(!desc) {
+        librdf_log(world, 0, LIBRDF_LOG_ERROR, LIBRDF_FROM_PARSER, NULL,
+                   "Failed to find any Raptor parsers - Raptor may not be initialised correctly");
+        break;
+      }
     }
+
     syntax_name = desc->names[0];
     syntax_label = desc->label;
     if(desc->mime_types)
