@@ -84,11 +84,11 @@ static void
 librdf_free_serializer_factory(librdf_serializer_factory *factory) 
 {
   if(factory->name)
-    LIBRDF_FREE(cstring, factory->name);
+    LIBRDF_FREE(char*, factory->name);
   if(factory->label)
-    LIBRDF_FREE(cstring, factory->label);
+    LIBRDF_FREE(char*, factory->label);
   if(factory->mime_type)
-    LIBRDF_FREE(cstring, factory->mime_type);
+    LIBRDF_FREE(char*, factory->mime_type);
   if(factory->type_uri)
     librdf_free_uri(factory->type_uri);
   LIBRDF_FREE(librdf_serializer_factory, factory);
@@ -130,18 +130,17 @@ librdf_serializer_register_factory(librdf_world *world,
       goto oom;
   }
 
-  serializer=(librdf_serializer_factory*)LIBRDF_CALLOC(librdf_serializer_factory, 1,
-                                                       sizeof(librdf_serializer_factory));
+  serializer = LIBRDF_CALLOC(librdf_serializer_factory*, 1, sizeof(*serializer));
   if(!serializer)
     goto oom;
 
-  serializer->name=(char*)LIBRDF_MALLOC(cstring, strlen(name)+1);
+  serializer->name = LIBRDF_MALLOC(char*, strlen(name) + 1);
   if(!serializer->name)
     goto oom_tidy;
   strcpy(serializer->name, name);
 
   if(label) {
-    serializer->label=(char*)LIBRDF_MALLOC(cstring, strlen(label)+1);
+    serializer->label = LIBRDF_MALLOC(char*, strlen(label) + 1);
     if(!serializer->label)
       goto oom_tidy;
     strcpy(serializer->label, label);
@@ -149,7 +148,7 @@ librdf_serializer_register_factory(librdf_world *world,
 
   /* register mime type if any */
   if(mime_type) {
-    serializer->mime_type=(char*)LIBRDF_MALLOC(cstring, strlen(mime_type)+1);
+    serializer->mime_type = LIBRDF_MALLOC(char*, strlen(mime_type) + 1);
     if(!serializer->mime_type)
       goto oom_tidy;
     strcpy(serializer->mime_type, mime_type);
@@ -278,18 +277,21 @@ librdf_serializer_enumerate(librdf_world* world,
                         const char **name, const char **label)
 {
   librdf_serializer_factory *factory;
+  int ioffset = LIBRDF_GOOD_CAST(int, counter);
   
   librdf_world_open(world);
 
-  factory=(librdf_serializer_factory*)raptor_sequence_get_at(world->serializers,
-                                                         counter);
+  factory = (librdf_serializer_factory*)raptor_sequence_get_at(world->serializers,
+                                                               ioffset);
   if(!factory)
     return 1;
   
   if(name)
-    *name=factory->name;
+    *name = factory->name;
+
   if(label)
-    *label=factory->label;
+    *label = factory->label;
+
   return 0;
 }
 
@@ -405,11 +407,11 @@ librdf_new_serializer_from_factory(librdf_world *world,
 
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(factory, librdf_serializer_factory, NULL);
 
-  d=(librdf_serializer*)LIBRDF_CALLOC(librdf_serializer, 1, sizeof(librdf_serializer));
+  d = LIBRDF_CALLOC(librdf_serializer*, 1, sizeof(*d));
   if(!d)
     return NULL;
         
-  d->context=(char*)LIBRDF_CALLOC(serializer_context, 1, factory->context_length);
+  d->context = LIBRDF_CALLOC(void*, 1, factory->context_length);
   if(!d->context) {
     librdf_free_serializer(d);
     return NULL;
@@ -619,6 +621,7 @@ librdf_serializer_serialize_model_to_file(librdf_serializer* serializer,
  * @length_p: pointer to store length or NULL
  *
  * Write a #librdf_stream to a counted string.
+ * Caller should free the string with librdf_free_memory().
  * 
  * Return value: stream as string or NULL on failure
  **/
@@ -649,6 +652,7 @@ librdf_serializer_serialize_stream_to_counted_string(librdf_serializer* serializ
  * @length_p: pointer to store length or NULL
  *
  * Write a serialized #librdf_model to a counted string.
+ * The returned string must be freed by the caller using librdf_free_memory().
  * 
  * Return value: non 0 on failure
  **/
@@ -702,6 +706,7 @@ librdf_serializer_serialize_stream_to_string(librdf_serializer* serializer,
  * @model: the #librdf_model model to use
  *
  * Write a serialized #librdf_model to a string.
+ * The returned string must be freed by the caller using librdf_free_memory().
  * 
  * Return value: NULL on failure
  **/
@@ -915,7 +920,7 @@ struct log_data {
 } LogData;
 
 
-static int
+static int REDLAND_CALLBACK_STDCALL
 log_handler(void *user_data, librdf_log_message *message) 
 {
   struct log_data* ld=(struct log_data*)user_data;
@@ -940,7 +945,9 @@ log_handler(void *user_data, librdf_log_message *message)
 }
 
 
-#define EXPECTED_ERRORS 3
+#define EXPECTED_ERRORS1 3
+/* Extra error is another UTF-8 encoding error */
+#define EXPECTED_ERRORS2 4
 #define EXPECTED_WARNINGS 0
 
 #define SYNTAX_TYPE "ntriples"
@@ -1024,24 +1031,27 @@ main(int argc, char *argv[])
   string=librdf_serializer_serialize_model_to_counted_string(serializer,
                                                              base_uri, model,
                                                              &string_length);
-#define EXPECTED_BAD_STRING_LENGTH 382
-  if(string_length != EXPECTED_BAD_STRING_LENGTH) {
-    fprintf(stderr, "%s: Serialising model to RDF/XML returned string '%s' size %d, expected %d\n", program, string,
-            (int)string_length, EXPECTED_BAD_STRING_LENGTH);
+#define EXPECTED_BAD_STRING_LENGTH1 382
+/* Raptor 2.0.11 changed the serialization slightly */
+#define EXPECTED_BAD_STRING_LENGTH2 378
+  if(string_length != EXPECTED_BAD_STRING_LENGTH1 && 
+     string_length != EXPECTED_BAD_STRING_LENGTH2) {
+    fprintf(stderr, "%s: Serialising model to RDF/XML returned string '%s' size %d, expected %d or %d\n", program, string,
+            (int)string_length,
+            EXPECTED_BAD_STRING_LENGTH1, EXPECTED_BAD_STRING_LENGTH2);
     return 1;
   }
 
-  if(string)
-    free(string);
+  librdf_free_memory(string);
 
   librdf_free_uri(base_uri); base_uri=NULL;
   librdf_free_model(model); model=NULL;
   librdf_free_storage(storage); storage=NULL;
   
 
-  if(LogData.errors != EXPECTED_ERRORS) {
-    fprintf(stderr, "%s: Serialising to RDF/XML returned %d errors, expected %d\n", program,
-            LogData.errors, EXPECTED_ERRORS);
+  if(LogData.errors != EXPECTED_ERRORS1 && LogData.errors != EXPECTED_ERRORS2) {
+    fprintf(stderr, "%s: Serialising to RDF/XML returned %d errors, expected %d or %d\n", program,
+            LogData.errors, EXPECTED_ERRORS1, EXPECTED_ERRORS2);
     return 1;
   }
 
@@ -1090,8 +1100,7 @@ main(int argc, char *argv[])
   }
   librdf_free_stream(stream);
 
-  if(string)
-    free(string);
+  librdf_free_memory(string);
 
 
   fprintf(stderr, "%s: Serializing stream to a file handle\n", program);
