@@ -213,13 +213,13 @@ ltdl_module_callback(const char* filename, void* data)
        )
      || strchr(name, '-')
      ) {
-#if LIBRDF_DEBUG > 2
+#if defined(LIBRDF_DEBUG) && LIBRDF_DEBUG > 2
     LIBRDF_DEBUG3("not storage module file %s (%s)\n", name, filename);
 #endif
     return 0;
   }
 
-#if LIBRDF_DEBUG > 1
+#if defined(LIBRDF_DEBUG) && LIBRDF_DEBUG > 1
   LIBRDF_DEBUG3("LOADING storage module file %s (%s)\n", name, filename);
 #endif
 
@@ -280,9 +280,18 @@ librdf_storage_load_module(librdf_world *world,
   
   lt_dlhandle module = lt_dlopenext(lib_name);
 
-  if (module) {
+  if(module) {
+    const lt_dlinfo* info = lt_dlgetinfo(module);
+    
+    if(info->ref_count > 1) {
+      /* Already loaded so ignore */
+      lt_dlclose(module);
+      module = NULL;
+      return module;
+    }
+
     init = (init_func_t*)lt_dlsym(module, init_func_name);
-    if (init) {
+    if(init) {
       init(world);
     } else {
       LIBRDF_DEBUG2("Failed to initialize storage module %s\n", lib_name);
@@ -349,24 +358,35 @@ librdf_storage_register_factory(librdf_world* world,
       (storage=(librdf_storage_factory*)raptor_sequence_get_at(world->storages, i));
       i++) {
     if(!strcmp(storage->name, name)) {
+#if 1
+      /* Choosing to ignore this error since it probably is caused by
+       * scanning a directory of storage modules and finding a .la
+       * file (libtool) as well as the .so (module) and trying to
+       * load the same module twice.
+       *
+       * See bug http://bugs.librdf.org/mantis/view.php?id=460 for
+       * context
+       */
+      return 0;
+#else
       librdf_log(world,
                  0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
                  "storage %s already registered", storage->name);
       return 1;
+#endif
     }
   }
 
-  storage=(librdf_storage_factory*)LIBRDF_CALLOC(librdf_storage_factory, 1,
-                                                 sizeof(librdf_storage_factory));
+  storage = LIBRDF_CALLOC(librdf_storage_factory*, 1, sizeof(*storage));
   if(!storage)
     goto failed;
 
-  storage->name=(char*)LIBRDF_MALLOC(cstring, strlen(name)+1);
+  storage->name = LIBRDF_MALLOC(char*, strlen(name) + 1);
   if(!storage->name)
     goto tidy;
   strcpy(storage->name, name);
 
-  storage->label=(char*)LIBRDF_MALLOC(cstring, strlen(label)+1);
+  storage->label = LIBRDF_MALLOC(char*, strlen(label) + 1);
   if(!storage->label)
     goto tidy;
   strcpy(storage->label, label);
@@ -454,18 +474,21 @@ librdf_storage_enumerate(librdf_world* world,
                          const char **name, const char **label)
 {
   librdf_storage_factory *factory;
+  int ioffset = LIBRDF_GOOD_CAST(int, counter);
   
   librdf_world_open(world);
 
-  factory=(librdf_storage_factory*)raptor_sequence_get_at(world->storages,
-                                                          counter);
+  factory = (librdf_storage_factory*)raptor_sequence_get_at(world->storages,
+                                                            ioffset);
   if(!factory)
     return 1;
   
   if(name)
-    *name=factory->name;
+    *name = factory->name;
+
   if(label)
-    *label=factory->label;
+    *label = factory->label;
+
   return 0;
 }
 
@@ -499,7 +522,7 @@ librdf_new_storage(librdf_world *world,
   if(!factory) {
     librdf_log(world, 
                0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
-               "storage '%s' not found", name);
+               "storage '%s' not found", storage_name);
     return NULL;
   }
 
@@ -597,8 +620,7 @@ librdf_new_storage_from_storage(librdf_storage* old_storage)
     return NULL;
   }
 
-  new_storage=(librdf_storage*)LIBRDF_CALLOC(librdf_storage, 1,
-                                             sizeof(librdf_storage));
+  new_storage = LIBRDF_CALLOC(librdf_storage*, 1, sizeof(*new_storage));
   if(!new_storage)
     return NULL;
   
@@ -659,8 +681,7 @@ librdf_new_storage_from_factory(librdf_world *world,
     return NULL;
   }
   
-  storage=(librdf_storage*)LIBRDF_CALLOC(librdf_storage, 1,
-                                         sizeof(librdf_storage));
+  storage = LIBRDF_CALLOC(librdf_storage*, 1, sizeof(*storage));
   if(!storage) {
     librdf_free_hash(options);
     return NULL;
@@ -968,23 +989,13 @@ int
 librdf_storage_contains_statement(librdf_storage* storage,
                                   librdf_statement* statement) 
 {
-  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(storage, librdf_storage, 1);
+  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(storage, librdf_storage, 0);
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(statement, librdf_statement, 1);
 
-  /* subject can be a URI or blank node */
-  if(!statement->subject ||
-     (!librdf_node_is_resource(statement->subject) &&
-      !librdf_node_is_blank(statement->subject)))
-    return 1;
-  
-  /* predicate can only be a URI */
-  if(!statement->predicate || !librdf_node_is_resource(statement->predicate))
-     return 1;
-
-  if(!statement->object)
+  if(!librdf_statement_is_complete(statement))
     return 1;
 
-  return storage->factory->contains_statement(storage, statement);
+  return storage->factory->contains_statement(storage, statement) ? -1 : 0;
 }
 
 
@@ -1214,7 +1225,8 @@ librdf_storage_node_stream_to_node_create(librdf_storage* storage,
   if(!partial_statement)
     return NULL;
   
-  context=(librdf_storage_stream_to_node_iterator_context*)LIBRDF_CALLOC(librdf_storage_stream_to_node_iterator_context, 1, sizeof(librdf_storage_stream_to_node_iterator_context));
+  context = LIBRDF_CALLOC(librdf_storage_stream_to_node_iterator_context*, 1,
+                          sizeof(*context));
   if(!context) {
     librdf_free_statement(partial_statement);
     return NULL;
